/*
 * 	glacier-mkprofile.c - Create and modify the system profile
 *
 * 	This file is part of Glacier.
 *
 * 	Glacier is free software: you can redistribute it and/or modify it under the terms of the
 * 	GNU General Public License as published by the Free Software Foundation, either
 * 	version 3 of the License, or (at your option) any later version.
 *
 * 	Glacier is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * 	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * 	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License along with Glacier. If
 * 	not, see <https://www.gnu.org/licenses/>.
 */ 

#include <color.h>
#include <errno.h>
#include <libconfig.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <wchar.h>

#include <glacier_config.h>
#include <glacier_log.h>
#include <glacier_pkgops.h>
#include <glacier_runtime.h>

#include "glacier.h"

const wchar_t w_check = 0x2713;

int iprofile;

static void
usage(int argc, char *argv[])
{
	printf(COL_BLUE "[*] " COL_RESET "%s - create and modify the system profile\n", argv[0]);
	printf(COL_BLUE "[?] " COL_RESET "usage: %s [-h] [-v] [-f] [-u] [-x]\n", argv[0]);
	printf("\n");
	printf("%s {-h}			Show this message\n", argv[0]);
	printf("%s {-v} 			Show the current version\n", argv[0]);
	printf("%s {-l}			List available system profiles\n", argv[0]);
	printf("%s {-p} PROFILE		Create or update the system profile PROFILE at /etc/glacier/profile.cfg\n", argv[0]);
	printf("\n");
	printf("Glacier is free software.\n");
	printf("See the GNU GPL version 3 for details.\n");
}

static void
usage_small(int argc, char *argv[])
{
	printf(COL_RED "[x] " COL_RESET "usage: %s [-h] [-v] [-l] [-p]\n", argv[0]);
}

int
main(int argc, char *argv[])
{
	if (argc < 2) {
		printf(COL_RED "[x] " COL_RESET "%s: not enough arguments (expected 1, got %i)\n", argv[0], argc - 1);
		usage_small(argc, argv);
		exit(1);
	}

	setlocale(LC_CTYPE, "");

	int opt;

	while ((opt = getopt(argc, argv, "hvlp:")) != -1) {
	switch (opt) {
	case 'h':
		usage(argc, argv);
		return 0;
		break;
	case 'v':
		printf(GLACIER_VERSION "\n");
		return 0;
		break;
	case 'l':
		infolog("Available profiles:");
		printf("1)\tx86_64-musl\n");
		printf("\t");
		return 0;
		break;
	case 'p':
		if (argc < 3) { errlog("No system profile specified."); 
			return 1;
		}

		is_process_root("Cannot access the system profile: permission denied. Are you root?");

		if (strcmp(optarg, "x86_64-musl") == 0) {
			infolog ("Using system profile \"x86_64-musl\"");
			iprofile = 1;
		} else {
			fprintf(stderr, COL_RED "[x] " COL_RESET "System profile %s is unknown or not supported.\n", optarg);
			return 1;
		}

		if (access("/etc/glacier/profile.cfg", F_OK != 0)) {
			infolog("No profile exists, skipping backup...");
		} else if (access("/etc/glacier/profile.cfg", F_OK == 0) && access("/etc/glacier/profile.old", F_OK != 0)) {
			infolog("Creating backup of old system profile at /etc/glacier/profile.old...");

			if (remove("/etc/glacier/profile.old") == 0) {
				infolog("Removed old backup.");
			}

			FILE *oldprofile, *oldprofile_back;
			char filename[128], contents;
			oldprofile      = fopen("/etc/glacier/profile.cfg", "r");
			oldprofile_back = fopen("/etc/glacier/profile.old", "a+");
			contents = fgetc(oldprofile);

			while (contents != EOF) {
				fputc(contents, oldprofile_back);
				contents = fgetc(oldprofile);
			}

			fclose(oldprofile);
			fclose(oldprofile_back);

			successlog("Successfully created backup.");
		}

		infolog("Creating new profile configuration file...");
		static const char *newprofile = "/etc/glacier/profile.cfg";
		config_t pprofile;
		config_setting_t *setting, *root;

		config_init(&pprofile);

		root = config_root_setting(&pprofile);

		if (iprofile == 1) {
			infolog("Writing configuration to file...");

			setting = config_setting_add(root, "GLACIER_REPO", CONFIG_TYPE_STRING);
			config_setting_set_string(setting, "https://git.everestlinux.org/EverestLinux/epkgs-x86_64-musl");
			printf("\tGLACIER_REPO = \"https://git.everestlinux.org/EverestLinux/epkgs-x86_64-musl\";\n");

			setting = config_setting_add(root, "GLACIER_ARCH", CONFIG_TYPE_STRING);
			config_setting_set_string(setting, "x86_64");
			printf("\tGLACIER_ARCH = \"x86_64\";\n");

			setting = config_setting_add(root, "GLACIER_TARGET", CONFIG_TYPE_STRING);
			config_setting_set_string(setting, "x86_64-linux-musl");
			printf("\tGLACIER_TARGET = \"x86_64-linux-musl\";\n");

			setting = config_setting_add(root, "GLACIER_LOCALDB", CONFIG_TYPE_STRING);
			config_setting_set_string(setting, "/glacier/localdb/epkgs-x86_64-musl");
			printf("\tGLACIER_LOCALDB = \"/glacier/localdb/epkgs-x86_64-musl\";\n");

			setting = config_setting_add(root, "GLACIER_SYSTEM_PROFILE", CONFIG_TYPE_STRING);
			config_setting_set_string(setting, "x86_64-musl");
			printf("\tGLACIER_SYSTEM_PROFILE = \"x86_64-musl\";\n");
		}

		if (! config_write_file(&pprofile, newprofile)) {
			errlog("Error while applying new profile to configuration.");
			config_destroy(&pprofile);
			return(EXIT_FAILURE);
		}
	
		successlog("Successfully updated system profile.");
		config_destroy(&pprofile);
		return(EXIT_SUCCESS);
		break;
	case '?':
		printf(COL_RED "[x] " COL_RESET "%s: Unknown argument '%s'.\n", argv[0], argv[1]);
		usage_small(argc, argv);
		return 1;
		break;
	}

	}
	
	printf(COL_RED "[x] " COL_RESET "%s: Unknown argument '%s'.\n", argv[0], argv[1]);
	usage_small(argc, argv);
	return 1;
}
